[manpage_begin central n 1.0b3]
[require   Tcl 8.5]
[require   central [opt 1.0b3]]
[moddesc   {Central}]
[titledesc {Central}]
[category  {Programming tools}]
[copyright {2022 Stuart Cassoff <stwo@users.sourceforge.net>}]
[keywords  central pkgconfig]

[description]

	Central is a passive registration system for Tcl extensions and programs,
	based on the idea and workings of the Tcl [cmd {pkgconfig}] command.
	[para]
	To participate, a [cmd {pkgconfig}] command should exist in the namespace
	of an extension or program ([term {"package"}]).
	Central itself is not needed by a providing package, only by a querying program.
	[para]
	Central can be used by a program to scan the namespace hierarchy ([cmd {discover}]),
	and query any package information found. A Central package is the fully-qualified
	namespace inside which a pkgconfig command was found.
	[para]
	The [cmd {pkgconfig}] command provided by a package should
	behave like the Tcl [cmd {pkgconfig}] command.
	[para]
	Central makes no effort to interpret the information returned by a
	[cmd {pkgconfig}] command; this information is entirely up to the package.
	[para]
	Central is all about "about": allowing a program to easily
	provide information about itself to the outside world.
	[para]
	The usage example files, the program centraltool.tcl, and
	the Central extension itself contain [cmd {pkgconfig}] commands and other
	code bits that can be copied or serve as examples and should make it easy
	to add Central functionality to extensions or programs.
	[para]
	Central works with Jim as well as with Tcl.


[section {CENTRALIZED PACKAGES}]

	A centralized package connects a Tcl package name and a namespace.
	[para]
	A package is considered "centralized" if it has both "name" and "importfrom" keys.
	The "name" should be the name of the Tcl package used in [lb]package require[rb],
	and "importfrom" should be the namespace where procedures exported by the package can be found.
	The "importfrom" namespace will be used in [lb]namespace import[rb] invocations.
	If "importfrom" is empty then the package is still considered to be centralized.


[section {PACKAGE INITIALIZATION}]
	A package can declare an initialization command by having both "name" and "initializer" keys.
	A package that declares both keys can be used with the
	[lb][cmd {central initializer}][rb] and [lb][cmd {central initialize}][rb] commands.
	[para]
	If the value of the "initializer" key begins with a colon then the package initialization command
	will be the value of the "initializer" key.
	If the package has an "importfrom" key then the initialization command will be
	the concatenation of the value of the "importfrom" key, two colons, and the value of the "initializer" key.
	Otherwise, the package is not considered to have an initialization command.


[section {COMMANDS}]

[usage namespace import ::central::central]

[list_begin definitions]

	[call [cmd {central discover}] [opt [arg {exclude}]]]
		Discovers packages. Ignores packages specified in [arg {exclude}].
		Should be called before calling other Central commands.
		Returns a list of all discovered packages.

	[call [cmd {central packages}]]
		Returns a list of previously discovered packages.

	[call [cmd {central packagekeys}] [arg {package}] [opt [arg {key}]]]
		Returns a list of all keys for package [arg {package}] matching [arg {key}].
		If [arg {key}] isn't supplied then the default pattern "*" will be used for [arg {key}].

	[call [cmd {central package}] [opt [arg {package}]] [opt [arg {key}]]]
		Returns all package info or info matching [arg {package}] and [arg {key}].
		If [arg {package}] isn't supplied then the default pattern "*" will be used for [arg {package}].
		If [arg {key}] isn't supplied then the default pattern "*" will be used for [arg {key}].

	[call [cmd {central find}] [opt [arg {key}]] [opt [arg {value}]]]
		Finds packages.
		Returns the package if [arg {key}] and [arg {value}] do not contain glob chars
		and only one match was found, otherwise returns a list of {package key value} lists for all matches.
		If [arg {key}] isn't supplied then the default pattern "*" will be used for [arg {key}].
		If [arg {value}] isn't supplied then the default pattern "*" will be used for [arg {value}].

	[call [cmd {central centralized}] [arg {tclpackage}]]
		Returns a 2-element list of {package namespace} if the Tcl package is centralized,
		the 2-element list {{} {}} otherwise.

	[call [cmd {central exports}] [arg {tclpackage}]]
		Returns a list of procedures exported by centralized Tcl package [arg {tclpackage}]
		or an empty list if [arg {tclpackage}] isn't a centralized package.

	[call [cmd {central import}] [opt [arg {-force}]] [arg {tclpackage}] [opt [arg {pattern ...}]]]
		Imports procedures from centralized Tcl package [arg {tclpackage}].
		If [arg {tclpackage}] is a centralized package, "importfrom" isn't empty,
		and the invocation of [lb]namespace import[rb] was successful,
		then the return value will be a list of attempted import patterns,
		otherwise an empty list is returned.
		The [arg {-force}] option can be used to import commands over existing commands.

	[call [cmd {central initializer}] [arg {tclpackage}]]
		Returns the initialization command for [arg {tclpackage}]
		or an empty string if [arg {tclpackage}] doesn't exist or doesn't have an initialization command.

	[call [cmd {central initialize}] [arg {tclpackage}] [opt [arg {arg ...}]]]
		Calls the initialization command for [arg {tclpackage}] in the callers environment,
		passing any [opt [arg {arg ...}]]. An error is returned if the [arg {tclpackage}]
		doesn't exist or doesn't have an initialization command, otherwise the returned
		value of the initialization command is returned.

	[call [cmd {central require}] [opt [arg {-exact}]] \
		[opt [arg {-noinit}]] [opt [arg {-noimport}]] \
		[opt [arg {-init command}]] [opt [arg {-import {command ...}}]] \
		[opt [arg {-allinfo}]] [arg {tclpackage}] [opt [arg {requirement ...}]]]
		In the callers namespace, performs [lb]package require [opt [arg {-exact}]] [arg {tclpackage}] [opt [arg {requirement ...}]][rb],
		[lb]central discover[rb], [lb]central initialize [arg {tclpackage}][rb], and [lb]central import [arg {tclpackage}][rb].
		The options [arg {-noinit}] and [arg {-noimport}] disable all initialization and importing, respectively.
		The [arg {-init command}] option specifies a command to call instead of any package initialization command.
		The supplied initialization command will be called with one argument: the initialization command of the package.
		The initialization info will be set to the return value of the supplied initialization command.
		The [arg {-import {command ...}}] option can be used to specify a list of commands to import.
		If the [arg {-allinfo}] option is specified then a dictionary of require, init, and import info is returned,
		otherwise, the returned value is the returned value of the Tcl [lb]package require[rb] command.
		A package must have the keys "name", "initializer", and "importfrom" for this to work.

[list_end]


[section EXAMPLES]

[example {
	# Discover packages and display package info.

	package require central
	namespace import ::central::central
	central discover
	foreach package [central packages] {
		foreach key [central packagekeys $package] {
			puts [string cat $key " -> " \\
				[central package $package $key]]
		}
	}
}]

[example {
	# Find and use a package.

	package require central
	namespace import ::central::central

	package require myPackage

	central discover

	set package [central find name myPackage]

	if {[regexp {^.+\db\d+$} [central package $package version]} {
		puts "Still in beta!"
	}


	# Find all packages with ISC licenses,
	  for all packages that have a key "license,type".

	central find license,type ISC
}]

[example {
	# Using a centralized package.

	package require central
	namespace import ::central::central

	package require MyPackage

	central discover

	puts "MyPackage exports: [central exports MyPackage]"

	# namespace import a few commands
	#
	central import MyPackage aCmd anotherCmd z*

	# namespace import *
	#
	central import MyPackage
}]


[see_also central::extras Tcl_RegisterConfig(3)]


[manpage_end]
